/*****************************************************************************/
/*                                                                           */
/*	 MJDate.c                                                                */
/*                                                                           */
/*   Simple Win32 program to display MJD calendar.                           */
/*   Uses dialog box as its main window.                                     */
/*   Has no menu, toolbuttons or other windows.                              */
/*                                                                           */
/*   Uses MJDate2.c for dialog box procedure                                 */
/*   Uses MJDate3.c for date functions                                       */
/*                                                                           */
/*   Major revision record:                                                  */
/*      05/29/99    Created from Simple.c                                    */
/*		05/30/99	Added calendar functionality from Stable32               */
/*		05/31/99	Added new icon, bitmap, colors & other decorations       */
/*		06/01/99	Added minimize button                                    */
/*		08/13/02	Added autorepeat for year & month up & down buttons      */
/*					Eliminated Datum bitmap                                  */
/*		            Eliminated large month font                              */
/*		08/19/02	Added title & title icon                                 */
/*		11/25/06	Version 2.0 - Uses HTML help system: MJDate.chm          */
/*		01/03/07	Changed to Version 2.1 in help file                      */
/*		12/29/07	Added SHOWMJD and SHOWGPS macros                         */
/*					May create custom version w/o GPS week # & MJD displays  */
/*		04/10/08	Added display of current MJD when minimized              */
/*					Changed version to 2.2 in help file                      */
/*                                                                           */
/* (c) 1999-2008 W. Riley  Hamilton Technical Services  All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

// Includes
#include "MJDate.h"
#include "resource.h"

// Global variables
// Note: All string allocations have extra character for EOS
HWND hwndMain;								// Main window handle
HINSTANCE hInst;							// Application instance
HICON hIcon;								// Application icon
HICON hIcon2;								// Title icon
HANDLE hAccel;								// Handle for accelerators
char szTrace[BUFFER_SIZE+1];				// Trace buffer
BOOL bAccel=FALSE;							// Accelerator key flag

// WinMain() function
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	PSTR szCmdLine, int nCmdShow)
{
	// Local variables
    MSG         msg;						// Main message loop message

	// Get handle to application icon
	hIcon=LoadIcon(hInstance, "MJDateIcon");

	// Get handle to title icon
	hIcon2=LoadIcon(hInstance, "MJDateIcon2");

	// Set global instance handle
	hInst=hInstance;

	// Create dialog box to be used as main window
	hwndMain=CreateDialog(hInstance, "MJDateBox", NULL,	MJDateDlgProc);
	
	// Load logo bitmap
	// hLogoBitmap=LoadBitmap(hInstance, "LOGO");

	// Load accererators
	hAccel=LoadAccelerators(hInstance, "MJDateAccel");

	// Display main window
    ShowWindow(hwndMain, nCmdShow);
    UpdateWindow(hwndMain);

	// Main message loop
    while(GetMessage(&msg, NULL, 0, 0))
    {
		// Is this an accelerator message?
		// Alt-x or Alt-X will close program, F1 will invoke help
		// Page up or down will advance or retard year
		// Up or down arrow will advance or retard month
		// Home will go to current date
		// Esc will also close program, but is not accelerator
		// Also right & left arrows will move up & down thru buttons
		// and spacebar will press them
		// And, of course, there are Alt-letter button shortcut keys 
		if(!TranslateAccelerator(hwndMain, hAccel, &msg))
		{
			// Is this a dialog box message?
			if(IsDialogMessage(hwndMain, &msg))
			{
				// If so, skip normal message handling
				continue;
			}
		
			// Otherwise process message as normal
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			// Set accelerator flag
			bAccel=TRUE;
		}
    }

    // Free Resources
	// DeleteObject(hLogoBitmap);

	return msg.wParam;
}

/*****************************************************************************/
