/*****************************************************************************/
/*                                                                           */
/*                             MJDate.h                                      */
/*                                                                           */
/*      Simple header file                                                   */
/*                                                                           */
/*      Revision record:                                                     */
/*          05/29/99    Created from MJDate.h                                */
/*			11/25/06	Added htmlhrlp.h for HTLM help support               */
/*			12/29/07	Added SHOWMJD and SHOWGPS macros                     */
/*                                                                           */
/*  (c) 1999-2006  W. Riley Hamilton Technical Services All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

// Include Windows header files
#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include "htmlhelp.h" // Copied to local folder along with htmlhelp.lib
// #include <commctrl.h>

// Include standard C headers
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <commdlg.h>
#include <direct.h>
#include <math.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <dos.h>

// Macro to control MJD display
// Unfefine this to remove MJD from calendar
#define SHOWMJD

// Macro to control GPS week # display
// Unfefine this to remove GPS week # from calendar
#define SHOWGPS

// Debugging header and macros
// Note: Define NDEBUG to disable asserts
//#define NDEBUG
#include <assert.h>

// Assert macro Ref: DDJ Feb 98 p.10
// Does nothing if x is TRUE or if NDEBUG is defined
// If x is FALSE, execution immediately stops and debugger opens at ASSERT line
#ifdef NDEBUG
    #define ASSERT(x)
#else
    #define ASSERT(x) if(x);else{*((void**)0)=0;}
#endif

// Macro to simplify getting handle to dialog box control
// Note: 1st arg of dialog box proc MUST be named hDlg
#define GETHCTL(id) GetDlgItem(hDlg, (id))

// Trace, debug & display macros
// Require global variables char szTrace[BUFFER_SIZE+1} and HWND hwndMain

// Trace int
// Usage  TRACE_INT(nTrace); where nTrace is int variable to be displayed
#define TRACE_INT(var) sprintf(szTrace, #var "=%d", var); \
    MessageBox(hwndMain, szTrace, "Trace Int", MB_OK)

// Debug int
// Usage  DEBUG_INT(nTrace); where nTrace is int variable to be displayed
#define DEBUG_INT(var) sprintf(szTrace, "\n" #var "=%d", var); \
    OutputDebugString(szTrace)

// Display int
// Usage  DISPLAY_INT(nTrace); where nTrace is int variable to be displayed
#define DISPLAY_INT(var) sprintf(szTrace, "int " #var "=%d", var); \
    DebugMessageBox(hwndMain, __FILE__, __LINE__, szTrace)

// Trace unsigned int
// Usage  TRACE_UINT(uTrace); where uTrace is unsigned int variable to disp
#define TRACE_UINT(var) sprintf(szTrace, #var "=%u", var); \
    MessageBox(hwndMain, szTrace, "Trace Unsigned Int", MB_OK)

// Debug unsigned int
// Usage  DEBUG_UINT(uTrace); where uTrace is uint variable to be displayed
#define DEBUG_UINT(var) sprintf(szTrace, "\n" #var "=%u", var); \
    OutputDebugString(szTrace)

// Display unsigned int
// Usage  DISPLAY_UINT(uTrace); where uTrace is uint variable to be displayed
#define DISPLAY_UINT(var) sprintf(szTrace, "uint " #var "=%u", var); \
    DebugMessageBox(hwndMain, __FILE__, __LINE__, szTrace)

// Trace long
// Usage  TRACE_LONG(lnTrace); where lnTrace is long int to be displayed
#define TRACE_LONG(var) sprintf(szTrace,#var "=%ld",var); \
    MessageBox(hwndMain, szTrace, "Trace Long", MB_OK)

// Debug long
// Usage  DEBUG_LONG(lnTrace); where lnTrace is long variable to be displayed
#define DEBUG_LONG(var) sprintf(szTrace, "\n" #var "=%ld", var); \
    OutputDebugString(szTrace)

// Display long
// Usage  DISPLAY_LONG(lnTrace); where lnTrace is long variable to be displayed
#define DISPLAY_LONG(var) sprintf(szTrace, "long " #var "=%ld", var); \
    DebugMessageBox(hwndMain, __FILE__, __LINE__, szTrace)

// Trace real (float or double)
// Usage  TRACE_EXP(fTrace); where fTrace is float or double to be displayed
#define TRACE_EXP(var) sprintf(szTrace,#var "=%e",var); \
    MessageBox(hwndMain, szTrace, "Trace Exp", MB_OK)

// Debug real
// Usage  DEBUG_EXP(fTrace); where fTrace is real variable to be displayed
#define DEBUG_EXP(var) sprintf(szTrace, "\n" #var "=%e", var); \
    OutputDebugString(szTrace)

// Display real
// Usage  DISPLAY_EXP(fTrace); where fTrace is real variable to be displayed
#define DISPLAY_EXP(var) sprintf(szTrace, "real " #var "=%e", var); \
    DebugMessageBox(hwndMain, __FILE__, __LINE__, szTrace)

// Trace address (pointer)
// Usage  TRACE_ADR(pTrace); where pTrace is pointer to be displayed
#define TRACE_ADR(var) sprintf(szTrace, #var "=%p", var); \
    MessageBox(hwndMain, szTrace, "Trace Adr", MB_OK)

// Debug pointer
// Usage  DEBUG_ADR(pTrace); where pTrace is pointer variable to be displayed
#define DEBUG_ADR(var) sprintf(szTrace, "\n" #var "=%p", var); \
    OutputDebugString(szTrace)

// Display int
// Usage  DISPLAY_ADR(pTrace); where pTrace is pointer variable to be displayed
#define DISPLAY_ADR(var) sprintf(szTrace, "pointer " #var "=%p", var); \
    DebugMessageBox(hwndMain, __FILE__, __LINE__, szTrace)

// Trace string
// Usage  TRACE_STR(szTrace); where szTrace is string to be displayed
#define TRACE_STR(var) MessageBox(hwndMain, var, "Trace Str", MB_OK)

// Debug string
// Usage  DEBUG_STR(szTrace); where szTrace is string variable to be displayed
#define DEBUG_STR(var) OutputDebugString(var)

// Display string
// Usage  DISPLAY_STR(szTrace); where szTrace is string var to be displayed
#define DISPLAY_STR(var) DebugMessageBox(hwndMain, __FILE__, __LINE__, var)

// Error string
// Usage  ERROR_STR(szTrace); where szTrace is error message to be displayed
#define ERROR_STR(var) MessageBox(hwndMain, var, "Simple Error", \
	MB_ICONEXCLAMATION)

// Data type
#define F_TYPE double

// Buffer sizes
#define BUFFER_SIZE          128
#define FILENAME_SIZE        256

// Bitmap IDs
// Note: Do NOT include a # define for a bitmap that is accessed via a string name
// See "Win32 Programming", p. 336 for more info 		

// Calendar Dialog Box 1000-1019
#define IDC_CAL_CANCEL      1001
#define IDC_CAL_YRUP        1002
#define IDC_CAL_YRDN        1003
#define IDC_CAL_MOUP        1004
#define IDC_CAL_MODN        1005
#define IDC_CAL_TODAY       1006
#define IDC_CAL_MONTH       1007
#define IDC_CAL_YEAR        1008
#define IDC_CAL_GPS         1009
#define IDC_CAL_LOGO		1010
#define IDC_CAL_TITLE		1011
#define IDC_CAL_COPY		1012

// Calendar Dialog Box Days 1100-1149
#define IDC_CAL_DAY0        1100  // This is used to calc the actual ID #
#define IDC_CAL_DAY1        1101  // To do so, must keep the ID #s in order
#define IDC_CAL_DAY2        1102
#define IDC_CAL_DAY3        1103
#define IDC_CAL_DAY4        1104
#define IDC_CAL_DAY5        1105
#define IDC_CAL_DAY6        1106
#define IDC_CAL_DAY7        1107
#define IDC_CAL_DAY8        1108
#define IDC_CAL_DAY9        1109
#define IDC_CAL_DAY10       1110
#define IDC_CAL_DAY11       1111
#define IDC_CAL_DAY12       1112
#define IDC_CAL_DAY13       1113
#define IDC_CAL_DAY14       1114
#define IDC_CAL_DAY15       1115
#define IDC_CAL_DAY16       1116
#define IDC_CAL_DAY17       1117
#define IDC_CAL_DAY18       1118
#define IDC_CAL_DAY19       1119
#define IDC_CAL_DAY20       1120
#define IDC_CAL_DAY21       1121
#define IDC_CAL_DAY22       1122
#define IDC_CAL_DAY23       1123
#define IDC_CAL_DAY24       1124
#define IDC_CAL_DAY25       1125
#define IDC_CAL_DAY26       1126
#define IDC_CAL_DAY27       1127
#define IDC_CAL_DAY28       1128
#define IDC_CAL_DAY29       1129
#define IDC_CAL_DAY30       1130
#define IDC_CAL_DAY31       1131
#define IDC_CAL_DAY32       1132
#define IDC_CAL_DAY33       1133
#define IDC_CAL_DAY34       1134
#define IDC_CAL_DAY35       1135
#define IDC_CAL_DAY36       1136
#define IDC_CAL_DAY37       1137
#define IDC_CAL_DAY38       1138
#define IDC_CAL_DAY39       1139
#define IDC_CAL_DAY40       1140
#define IDC_CAL_DAY41       1141
#define IDC_CAL_DAY42       1142

// MJD in legend 1150
#define IDC_CAL_MJDLEGEND	1150

// Global variables
extern HWND hwndMain;						// Main window handle
extern HINSTANCE hInst;						// Application instance
extern HICON hIcon;							// Application icon
extern HBITMAP hLogoBitmap;					// Logo bitmap handle
extern char szTrace[BUFFER_SIZE+1];			// Buffer for trace macro
extern BOOL bAccel;							// Accelerator key flag

// Function prototypes
// MJDate.c - Main function
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	PSTR szCmdLine, int nCmdShow);

// MJDate2.c - Dialog box procedure
BOOL CALLBACK MJDateDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);
void WINAPI WriteCalendar(HWND hDlg, int nMonth, int nThisMonth,
	int nYear, int nThisYear, int nToday, BOOL bClip);
void WINAPI CopyCalendar(HWND hDlg);

// MJDate3.c - Date functions
void WINAPI BreakDate(LONG gdate, INT *mo,INT *da,INT *yr);
INT WINAPI DayOfWeek(LONG gdate);
LONG WINAPI DateToJulian(LONG gdate);
LONG WINAPI JulianToDate(LONG jdate);
LONG WINAPI MakeDate(INT mo, INT da, INT yr);
INT WINAPI MJDtoGPS(LONG MJD);
void WINAPI DateConv(int *lpnMonth, int *lpnDaysInMonth,
	int *lpnYear, int *lpnDOW, int *lpnDOY, long *lplnMJD);
void WINAPI DrawBitmap(HDC hdc, HBITMAP hBitmap, short xStart,
	short yStart);