/*****************************************************************************/
/*                                                                           */
/*   MJDate2.c                                                               */
/*                                                                           */
/*   Dialog box procedure for MJDate program                                 */
/*                                                                           */
/*   (c) 1999-2008 W. Riley Hamilton Technical Services All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

// Includes
#include "MJDate.h"

/*****************************************************************************/
/*                                                                           */
/*                   MJDate Dialog Procedure                                 */
/*                                                                           */
/*      Revision record:                                                     */
/*          05/29/99    Created from Simple2.c                               */
/*			05/30/99	Recreated from CalendarDlgProc of Stable32           */
/*			05/31/99	Added bitmap, colors & other decorations             */
/*						Added help file & accelerators                       */
/*			08/13/02	Added autorepeat for year & month up & down buttons  */
/*						Eliminated Datum logo                                */
/*						Eliminated multiple brush creation resource leak     */
/*						Eliminated large month font                          */
/*			08/19/02	Added Copy feature to copy calendar to clipboard     */
/*			11/25/06	Changed to HTML help system.                         */
/*			04/10/08	Added display of current MJD when minimized          */
/*                                                                           */
/*****************************************************************************/

BOOL CALLBACK MJDateDlgProc(HWND hDlg, UINT message, WPARAM wParam,
    LPARAM lParam)
{
	#include <time.h>           // Header for Std C time functions & tm struct

	// Autorepeat macro
	#define SLOW 10				// Autorepeat accelerator slow threshold
	#define FAST 30				// Autorepeat accelerator fast threshold

    // Local variables
    RECT rect;                  // Calendar screen rectangle
	static HBRUSH hWBrush;		// White brush for coloring dlg box controls
	static HBRUSH hBBrush;		// Button brush for coloring dlg box controls
	// HDC hdc;					// DC handle
	// HFONT hFont;				// Font 
	static LOGFONT logfont;		// Font structure - static fills it it with zeros
	static struct tm *datetime; // Standard C time structure
                                // Day of Month=tm_mday (1-31)
                                // Month of Year=tm_mon (0-11)
                                // Year since 1900=tm_year
    char szTitle[BUFFER_SIZE+1];// Dialog box title
    static int nMonth;          // Month of year(1-12)
    static int nThisMonth;      // Current month
    static int nYear;           // Year
    static int nThisYear;       // Current year
    static int nToday;          // Today's day # (1-31)
    static long lTime;          // Time value
	static BOOL bClip=FALSE;	// Flag to copy calendar to clipboard
	static BOOL bButton=FALSE;	// Flag that autorepeating button active

	// Autorepeat variables
	static int bAuto;			// Flag for autorepeat
	static int nAccel;			// Autorepeat accelerator count				

	// MJD variables
	double mjd, mjd_add, mjd_div, nsec;

	// Peek message
	MSG msg;					// Message structure for local peek

    switch(message)
    {
        case WM_INITDIALOG:
        {
			// Create brushs to set background colors of non-text area
			hWBrush=CreateSolidBrush(0x00FFFFFF);
			hBBrush=CreateSolidBrush(GetSysColor(COLOR_BTNFACE));

			// Set the application system menu icon
			SetClassLong(hDlg, GCL_HICON, (LONG)hIcon);

			// Post message to load logo bitmap
			// PostMessage(hDlg, IDC_CAL_LOGO, 0, 0);

			// Get current date and time using Std C lib finctions
            time(&lTime);
            datetime=localtime(&lTime);
			nToday=datetime->tm_mday;
            nMonth=datetime->tm_mon+1;
            nThisMonth=nMonth;
            nYear=datetime->tm_year+1900;
            nThisYear=nYear;

			#if(1) // New code 04/10/08
			// Activate a 1-second timer
			SetTimer(hDlg, 1, 1000, NULL);
			#endif

			// Write calendar
            WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear, nToday, bClip);
        }
        return TRUE;

		#if(1) // New code 04/11/08
		case WM_ACTIVATE:
        {
			// Check if window is minimized
			if(IsIconic(hDlg))
			{
				// Get current MJD
				nsec = (double) time(NULL);
				mjd_add = 40587.0;
				mjd_div = 86400.0;
				mjd = mjd_add + nsec/mjd_div;

				// Show current MJD if window minimized
				// Just show integer MJD
				// Because value does not update
				sprintf(szTitle, "MJD=%d", (int)mjd);
				SetWindowText(hDlg, szTitle);
			}
			else
			{
				// Show program name if window is not minimized
				SetWindowText(hDlg, "MJDate");

				// This updates calendar to current day
				// when displaying it again after being minimized
				// Get current date and time using Std C lib finctions
				time(&lTime);
				datetime=localtime(&lTime);
				nToday=datetime->tm_mday;
				nMonth=datetime->tm_mon+1;
				nThisMonth=nMonth;
				nYear=datetime->tm_year+1900;
				nThisYear=nYear;

				// Write calendar
				WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear, nToday, bClip);
			}
        }
        break;
		#endif

		#if(1) // New code 04/10/08
		case WM_TIMER:
        {
			// Check if window is minimized
			if(IsIconic(hDlg))
			{
				#if(0) // For testing
				// Make a beep
				MessageBeep(MB_OK);
				#endif

				// Get current UTC MJD
				// The time function returns the number of seconds elapsed since midnight
				// (00:00:00:), January 1, 1970 UTC according to the system clock
				// It take into effect the local time zone setting
				// so that, for example, if the local time is EST, the UTC returned
				// by time() is 5 hours ahead.
				// The timer is not synchronized to the exact second
				nsec = (double) time(NULL);
				mjd_add = 40587.0;
				mjd_div = 86400.0;
				mjd = mjd_add + nsec/mjd_div;

				// Show current MJD if window minimized
				// Just show integer MJD
				// Because value does not update
				sprintf(szTitle, "MJD=%f", mjd);
				SetWindowText(hDlg, szTitle);
			}
			else
			{
				// Show program name if window is not minimized
				SetWindowText(hDlg, "MJDate");
			}
        }
        break;
		#endif

		// Set control colors				
		case WM_CTLCOLORSTATIC: // Msg sent by read-only edit & static controls
		{
			// Force gps week control colors - blue text on grey background
			if(GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_GPS))
			{
				// Set text colors - background and text itself
				SetBkColor(GET_WM_CTLCOLOR_HDC(wParam, lParam, message),
					GetSysColor(COLOR_BTNFACE));
				SetTextColor(GET_WM_CTLCOLOR_HDC(wParam, lParam, message),
					0x00800000); // Dark Blue

				return((DWORD)hBBrush);
			}

			// Force title control colors - red text on grey background
			if(GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_TITLE))
			{
				// Set colors - background and text itself
				SetBkColor(GET_WM_CTLCOLOR_HDC(wParam, lParam, message),
					GetSysColor(COLOR_BTNFACE));
				SetTextColor(GET_WM_CTLCOLOR_HDC(wParam, lParam, message),
					0x000000FF); // Red

				return((DWORD)hBBrush);
			}

			// Force logo control colors - blue text on grey background
			if(GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_LOGO))
			{
				// Set text colors - background and text itself
				SetBkColor(GET_WM_CTLCOLOR_HDC(wParam, lParam, message),
					GetSysColor(COLOR_BTNFACE));
				SetTextColor(GET_WM_CTLCOLOR_HDC(wParam, lParam, message),
					0x00800000); // Dark Blue

				return((DWORD)hBBrush);
			}

			// Force day control colors - black text on white background
			if((GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+0)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+1)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+2)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+3)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+4)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+5)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+6)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+7)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+8)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+9)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+10)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+11)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+12)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+13)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+14)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+15)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+16)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+17)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+18)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+19)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+20)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+21)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+22)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+23)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+24)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+25)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+26)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+27)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+28)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+29)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+30)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+31)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+32)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+33)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+34)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+35)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+36)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+37)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+38)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+39)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+40)) ||
			   (GET_WM_CTLCOLOR_HWND(wParam, lParam, message)==
				GETHCTL(IDC_CAL_DAY1+41)) )
			{
				// Set colors - background and text itself
				SetBkColor(GET_WM_CTLCOLOR_HDC(wParam, lParam, message),
					0x00FFFFFF);
				SetTextColor(GET_WM_CTLCOLOR_HDC(wParam, lParam, message),
					0x00000000); // Black

				return((DWORD)hWBrush);
			}
		}

		case WM_COMMAND:
        {
            switch(GET_WM_COMMAND_ID(wParam, lParam))
            {
                // Respond to help button
				case IDHELP:
                {
					#if(0)// Original Winhelp
                    WinHelp(hDlg, "mjdate.hlp", HELP_CONTENTS,
                        0L);
					#endif

					#if(1)// New HTML help 11/25/06
                    HtmlHelp(hDlg, "mjdate.chm", HH_DISPLAY_TOPIC,
                        0L);
					#endif
                }
                break;

                case IDC_CAL_YRUP:
                {
					// Respond to accelerator key
					if(bAccel)
					{
						// Reset accelerator flag
						bAccel=FALSE;
					
						// Increment year
						nYear++;

						// Limit max year to 2099
						if(nYear>2099)
						{
							nYear=1900;
						}

						// Rewrite calendar
						WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
							nToday, bClip);

						// Redisplay dialog box
						GetClientRect(hDlg, &rect);
						InvalidateRect(hDlg, &rect, FALSE);
					}

					// Start when button depressed
					if((GET_WM_COMMAND_CMD(wParam, lParam))==BN_SETFOCUS)
					{	
						// Set button to pressed state
						Button_SetState(GETHCTL(IDC_CAL_YRUP), 1);
						
						// Activate autorepeating
						bAuto=TRUE;

						// Set autorepeating button flag
						bButton=TRUE;

						// Set acceperator count to zero
						nAccel=0;

						// Autorepeat loop
						while(bAuto)
						{
							// Increment year
							nYear++;

							// Limit max year to 2099
							if(nYear>2099)
							{
								nYear=1900;
							}

							// Rewrite calendar
							WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
								nToday, bClip);

							// Redisplay dialog box
							GetClientRect(hDlg, &rect);
							InvalidateRect(hDlg, &rect, FALSE);

							// Let system update display & get messages
							while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
							{
								if(!IsDialogMessage(hDlg, &msg))
								{
									TranslateMessage(&msg);
									DispatchMessage(&msg);
								}
							}

							// Increment accelerator count
							nAccel++;
							
							// Set autorepeat rate
							if(nAccel<SLOW) // Slow
							{
								Sleep(200);
							}
							else if(nAccel<FAST) // Medium
							{
								Sleep(100);
							}
							else // Fast
							{
								Sleep(50);
							}
						}
					}

					// Stop when button released or respond to keyboard shortcut
					if((GET_WM_COMMAND_CMD(wParam, lParam))==BN_CLICKED)
					{	
						if(bButton) // Button released
						{
							// Stop autorepeating
							bAuto=FALSE;

							// Reset autorepeating button flag
							bButton=FALSE;
							
							// Remove the last increment to allow single steps
							// Decrement year
							nYear--;

							// Limit min year to 1900
							if(nYear<1900)
							{
								nYear=2099;
							}

							// Rewrite calendar
							WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
								nToday, bClip);

							// Redisplay dialog box
							GetClientRect(hDlg, &rect);
							InvalidateRect(hDlg, &rect, FALSE);

							// Remove focus from button
							SetFocus(GETHCTL(IDC_CAL_TODAY));
						}
						else // Respond to year up ALT-Y keyboard shortcut
						{
							// Increment year
							nYear++;

							// Limit max year to 2099
							if(nYear>2099)
							{
								nYear=1900;
							}

							// Rewrite calendar
							WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
								nToday, bClip);

							// Redisplay dialog box
							GetClientRect(hDlg, &rect);
							InvalidateRect(hDlg, &rect, FALSE);
						}
					}
				}
                break;

                case IDC_CAL_YRDN:
                {
					// Respond to accelerator key
					if(bAccel)
					{
						// Reset accelerator flag
						bAccel=FALSE;
					
						// Decrement year
						nYear--;

						// Limit min year to 1900
						if(nYear<1900)
						{
							nYear=2099;
						}

						// Rewrite calendar
						WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
							nToday, bClip);

						// Redisplay dialog box
						GetClientRect(hDlg, &rect);
						InvalidateRect(hDlg, &rect, FALSE);
					}

					// Start when button depressed
					if((GET_WM_COMMAND_CMD(wParam, lParam))==BN_SETFOCUS)
					{	
						// Set button to pressed state
						Button_SetState(GETHCTL(IDC_CAL_YRDN), 1);
						
						// Activate autorepeating
						bAuto=TRUE;

						// Set autorepeating button flag
						bButton=TRUE;

						// Set acceperator count to zero
						nAccel=0;

						// Autorepeat loop
						while(bAuto)
						{
							// Decrement year
							nYear--;

							// Limit min year to 1900
							if(nYear<1900)
							{
								nYear=2099;
							}

							// Rewrite calendar
							WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
								nToday, bClip);

							// Redisplay dialog box
							GetClientRect(hDlg, &rect);
							InvalidateRect(hDlg, &rect, FALSE);

							// Let system update display & get messages
							while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
							{
								if(!IsDialogMessage(hDlg, &msg))
								{
									TranslateMessage(&msg);
									DispatchMessage(&msg);
								}
							}

							// Increment accelerator count
							nAccel++;
							
							// Set autorepeat rate
							if(nAccel<SLOW) // Slow
							{
								Sleep(200);
							}
							else if(nAccel<FAST) // Medium
							{
								Sleep(100);
							}
							else // Fast
							{
								Sleep(50);
							}
						}
					}

					// Stop when button released or respond to keyboard shortcut
					if((GET_WM_COMMAND_CMD(wParam, lParam))==BN_CLICKED)
					{	
						if(bButton) // Button released
						{
							// Stop autorepeating
							bAuto=FALSE;

							// Reset autorepeating button flag
							bButton=FALSE;
							
							// Remove the last decrement to allow single steps
							// Increment year
							nYear++;

							// Limit max year to 2099
							if(nYear>2099)
							{
								nYear=1900;
							}

							// Rewrite calendar
							WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
								nToday, bClip);

							// Redisplay dialog box
							GetClientRect(hDlg, &rect);
							InvalidateRect(hDlg, &rect, FALSE);

							// Remove focus from button
							SetFocus(GETHCTL(IDC_CAL_TODAY));
						}
						else // Respond to year down ALT-R keyboard shortcut
						{
							// Decrement year
							nYear--;

							// Limit min year to 1900
							if(nYear<1900)
							{
								nYear=2099;
							}

							// Rewrite calendar
							WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
								nToday, bClip);

							// Redisplay dialog box
							GetClientRect(hDlg, &rect);
							InvalidateRect(hDlg, &rect, FALSE);
						}
					}
                }
                break;

                case IDC_CAL_MOUP:
				{
					// Respond to accelerator key
					if(bAccel)
					{
						// Reset accelerator flag
						bAccel=FALSE;

						// Increment month
						nMonth++;
						if(nMonth==13)
						{
							nMonth=1;
							nYear++;
						}

						// Limit max year to 2099
						if(nYear>2099)
						{
							nYear=1900;
						}

						// Rewrite calendar
						WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
							nToday, bClip);

						// Redisplay dialog box
						GetClientRect(hDlg, &rect);
						InvalidateRect(hDlg, &rect, FALSE);
					}

					// Start when button depressed
					if((GET_WM_COMMAND_CMD(wParam, lParam))==BN_SETFOCUS)
					{	
						// Set button to pressed state
						Button_SetState(GETHCTL(IDC_CAL_MOUP), 1);
						
						// Activate autorepeating
						bAuto=TRUE;

						// Set autorepeating button flag
						bButton=TRUE;

						// Set acceperator count to zero
						nAccel=0;

						// Autorepeat loop
						while(bAuto)
						{
							// Increment month
							nMonth++;
							if(nMonth==13)
							{
								nMonth=1;
								nYear++;
							}

							// Limit max year to 2099
							if(nYear>2099)
							{
								nYear=1900;
							}

							// Rewrite calendar
							WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
								nToday, bClip);

							// Redisplay dialog box
							GetClientRect(hDlg, &rect);
							InvalidateRect(hDlg, &rect, FALSE);

							// Let system update display & get messages
							while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
							{
								if(!IsDialogMessage(hDlg, &msg))
								{
									TranslateMessage(&msg);
									DispatchMessage(&msg);
								}
							}

							// Increment accelerator count
							nAccel++;
							
							// Set autorepeat rate
							if(nAccel<SLOW) // Slow
							{
								Sleep(200);
							}
							else if(nAccel<FAST) // Medium
							{
								Sleep(100);
							}
							else // Fast
							{
								Sleep(50);
							}
						}
					}

					// Stop when button released or respond to keyboard shortcut
					if((GET_WM_COMMAND_CMD(wParam, lParam))==BN_CLICKED)
					{	
						if(bButton) // Button released
						{
							// Stop autorepeating
							bAuto=FALSE;

							// Reset autorepeating button flag
							bButton=FALSE;
							
							// Remove the last increment to allow single steps
							// Decrement month
							nMonth--;
							if(nMonth==0)
							{
								nMonth=12;
								nYear--;
							}

							// Limit min year to 1900
							if(nYear<1900)
							{
								nYear=2099;
							}

							// Rewrite calendar
							WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
								nToday, bClip);

							// Redisplay dialog box
							GetClientRect(hDlg, &rect);
							InvalidateRect(hDlg, &rect, FALSE);

							// Remove focus from button
							SetFocus(GETHCTL(IDC_CAL_TODAY));
						}
						else // Respond to month up ALT-U keyboard shortcut
						{
							// Increment month
							nMonth++;
							if(nMonth==13)
							{
								nMonth=1;
								nYear++;
							}

							// Limit max year to 2099
							if(nYear>2099)
							{
								nYear=1900;
							}

							// Rewrite calendar
							WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
								nToday, bClip);

							// Redisplay dialog box
							GetClientRect(hDlg, &rect);
							InvalidateRect(hDlg, &rect, FALSE);
						}
					}
                }
				break;

                case IDC_CAL_MODN:
                {
					// Respond to accelerator key
					if(bAccel)
					{
						// Reset accelerator flag
						bAccel=FALSE;

						// Decrement month
						nMonth--;
						if(nMonth==0)
						{
							nMonth=12;
							nYear--;
						}

						// Limit min year to 1900
						if(nYear<1900)
						{
							nYear=2099;
						}

						// Rewrite calendar
						WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
							nToday, bClip);

						// Redisplay dialog box
						GetClientRect(hDlg, &rect);
						InvalidateRect(hDlg, &rect, FALSE);
					}

					// Start when button depressed
					if((GET_WM_COMMAND_CMD(wParam, lParam))==BN_SETFOCUS)
					{	
						// Set button to pressed state
						Button_SetState(GETHCTL(IDC_CAL_MODN), 1);
						
						// Activate autorepeating
						bAuto=TRUE;

						// Set autorepeating button flag
						bButton=TRUE;

						// Set acceperator count to zero
						nAccel=0;

						// Autorepeat loop
						while(bAuto)
						{
							// Decrement month
							nMonth--;
							if(nMonth==0)
							{
								nMonth=12;
								nYear--;
							}

							// Limit min year to 1900
							if(nYear<1900)
							{
								nYear=2099;
							}

							// Rewrite calendar
							WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
								nToday, bClip);

							// Redisplay dialog box
							GetClientRect(hDlg, &rect);
							InvalidateRect(hDlg, &rect, FALSE);

							// Let system update display & get messages
							while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
							{
								if(!IsDialogMessage(hDlg, &msg))
								{
									TranslateMessage(&msg);
									DispatchMessage(&msg);
								}
							}

							// Increment accelerator count
							nAccel++;
							
							// Set autorepeat rate
							if(nAccel<SLOW) // Slow
							{
								Sleep(200);
							}
							else if(nAccel<FAST) // Medium
							{
								Sleep(100);
							}
							else // Fast
							{
								Sleep(50);
							}
						}
					}

					// Stop when button released or respond to keyboard shortcut
					if((GET_WM_COMMAND_CMD(wParam, lParam))==BN_CLICKED)
					{	
						if(bButton) // Button released
						{
							// Stop autorepeating
							bAuto=FALSE;

							// Reset autorepeating button flag
							bButton=FALSE;
							
							// Remove the last decrement to allow single steps
							// Increment month
							nMonth++;
							if(nMonth==13)
							{
								nMonth=1;
								nYear++;
							}

							// Limit max year to 2099
							if(nYear>2099)
							{
								nYear=1900;
							}

							// Rewrite calendar
							WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
								nToday, bClip);

							// Redisplay dialog box
							GetClientRect(hDlg, &rect);
							InvalidateRect(hDlg, &rect, FALSE);

							// Remove focus from button
							SetFocus(GETHCTL(IDC_CAL_TODAY));
						}
						else // Respond to month down ALT-D keyboard shortcut
						{
							// Decrement month
							nMonth--;
							if(nMonth==0)
							{
								nMonth=12;
								nYear--;
							}

							// Limit min year to 1900
							if(nYear<1900)
							{
								nYear=2099;
							}


							// Rewrite calendar
							WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
								nToday, bClip);

							// Redisplay dialog box
							GetClientRect(hDlg, &rect);
							InvalidateRect(hDlg, &rect, FALSE);
						}
					}
                }
                break;

                case IDC_CAL_TODAY:
                {
                    // Redisplay dialog box
                    GetClientRect(hDlg, &rect);
                    InvalidateRect(hDlg, &rect, FALSE);

					// Reinitialize
					SendMessage(hDlg, WM_INITDIALOG, (WPARAM)0, (LPARAM)0);
				}
				break;

                case IDC_CAL_COPY:
                {
					// Set clipboard flag
					bClip=TRUE;

					// Rewrite calendar
					WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
						nToday, bClip);

                    // Redisplay dialog box
                    GetClientRect(hDlg, &rect);
                    InvalidateRect(hDlg, &rect, FALSE);

					// Let system update display & get messages
					while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
					{
						if(!IsDialogMessage(hDlg, &msg))
						{
							TranslateMessage(&msg);
							DispatchMessage(&msg);
						}
					}

					// Copy calendar bitmap to clipboard
					CopyCalendar(hDlg);

					// Clear clipboard flag
					bClip=FALSE;

					// Rewrite calendar - may need current day highlighted
					WriteCalendar(hDlg, nMonth, nThisMonth, nYear, nThisYear,
						nToday, bClip);

                    // Redisplay dialog box
                    GetClientRect(hDlg, &rect);
                    InvalidateRect(hDlg, &rect, FALSE);
				}
				break;

                case IDCANCEL:
                case IDC_CAL_CANCEL:
				{
					// Close program
					PostQuitMessage(0);
					return 0;
                }
                return TRUE;
            }
		}
        break;
     }
     return FALSE;
}

/*****************************************************************************/
/*                                                                           */
/*                   Calendar Subroutine                                     */
/*                                                                           */
/*  Function to write calendar info on screen for given month and year       */
/*                                                                           */
/*  Revision record:                                                         */
/*		05/31/99	Changed from separate month & year to title              */
/*		08/18/02	Added BOOL bClip parameter                               */
/*		12/29/07	Added SHOWMJD and SHOWGPS macro usage                    */
/*                                                                           */
/*****************************************************************************/

void WINAPI WriteCalendar(HWND hDlg, int nMonth, int nThisMonth,
	int nYear, int nThisYear, int nToday, BOOL bClip)
{
    // Local variables
    char szBuffer[BUFFER_SIZE+1];  // General text buffer
    char szTitle[BUFFER_SIZE+1];   // Title text buffer
    static char *szMonth[]={"January", "February", "March", "April", "May",
        "June", "July", "August", "September", "October", "November",
        "December"};            // Month names
    int i;                      // Index
	static int nDOW;            // Day of week (0-6, Sun=0)
    static int nDOY;            // Day of year
    static int nDaysInMonth;    // # days in month
    static long lnMJD;          // MJD

	// Get DOW, DOY and MJD
    DateConv(&nMonth, &nDaysInMonth, &nYear, &nDOW, &nDOY,
        &lnMJD);

	#ifdef SHOWGPS // Use this to control GPS week # display
	// Display GPS week #s
	i=MJDtoGPS(lnMJD);
	sprintf(szBuffer, "GPS Weeks %d-%d", i, ((i+5)%1024));

	// Limit display to + #s
	if(i>=0)
	{
		SetWindowText(GETHCTL(IDC_CAL_GPS), szBuffer);
	}
	else
	{
		SetWindowText(GETHCTL(IDC_CAL_GPS), "");
	}
	#endif

    // Compose month
    strcpy(szTitle, szMonth[nMonth-1]);

    // Compose year
    sprintf(szBuffer, "  %d", nYear);

	// Write title
	strcat(szTitle, szBuffer);
    SetWindowText(GETHCTL(IDC_CAL_TITLE), szTitle);
    
	// Clear cells
    for(i=0; i<42; i++)
    {
		sprintf(szBuffer, "");
        SetWindowText(GETHCTL(IDC_CAL_DAY1+i), szBuffer);
    }

	#ifdef SHOWMJD // Use this to include MJD in legend
    SetWindowText(GETHCTL(IDC_CAL_MJDLEGEND), "MJD");
	#else // Omit MJD in legend
    SetWindowText(GETHCTL(IDC_CAL_MJDLEGEND), "");
	#endif

    // Write to cells
    // Note that a CR LF is needed at the end of each line
    for(i=0; i<nDaysInMonth; i++)
    {
		#ifdef SHOWMJD // Use this to include MJD in calendar
		sprintf(szBuffer, "%2.0d   %d\r\n%ld", 1+i, nDOY+i,
            lnMJD+i);
		#else // Omit MJD in calendar
		sprintf(szBuffer, "%2.0d   %d\r\n", 1+i, nDOY+i);
		#endif

        SetWindowText(GETHCTL(
            IDC_CAL_DAY0+nDOW+1+i), szBuffer);
    }

    if(nYear==nThisYear && nMonth==nThisMonth && !bClip)
    {
        // Highlight current day
		SendMessage(GETHCTL(IDC_CAL_DAY0+nToday+nDOW), EM_SETSEL,
			GET_EM_SETSEL_MPS(0, 2));
	}

	return;
}

/*****************************************************************************/
/*                                                                           */
/*							CopyCalendar()                                   */
/*                                                                           */
/*		Function to copy calendar bitmap to Windows clipboard                */
/*		                                                                     */
/*                                                                           */
/*      Parameters:                                                          */
/*      	HWND hDlg	Handle to calendar dialog box                        */
/*                                                                           */
/*      Return:                                                              */
/*			None	(void)                                                   */
/*                                                                           */
/*		Revision record:                                                     */
/*			08/19/02	Created                                              */
/*                                                                           */
/*****************************************************************************/
void WINAPI CopyCalendar(HWND hDlg)
{
	// Local variables
	// Bitmap variables
	static HBITMAP hCalBitmap=0;	// Handle to calendar bitmap
    HDC hdcCalendar;				// Handle to calendar device context
	HDC hdcCalMem;					// Handle to calendar memory device context
    RECT rect;						// Calendar rectangle
	int xSize;						// Calendar width in pixels
	int ySize;						// Calendar height in pixels

	// Save calendar as color bitmap in memory
	// Get size of calendar dialog box
	GetClientRect(hDlg, &rect);
	xSize=rect.right-rect.left;
	ySize=rect.bottom-rect.top;

	// Use calendar portion only w/o buttons
	// Full XGA pixel width is 590, want left 485 pixels, so mult by 0.822
	// Ratio is screen resolution independent
	xSize*=822;
	xSize/=1000;

	// Get handle to calendar dialog device context
	hdcCalendar=GetDC(hDlg);

	// Create memory device context
	hdcCalMem=CreateCompatibleDC(hdcCalendar);

	// Detete existing calendar bitmap (if any)
	if(hCalBitmap)
	{
		DeleteObject(hCalBitmap);
	}

	// Create an empty device dependent color bitmap for calendar
	hCalBitmap=CreateCompatibleBitmap(hdcCalendar, xSize, ySize);

	// Select color bitmap into memory device context
	SelectObject(hdcCalMem, hCalBitmap);

	// Copy pixels from screen to memory
	BitBlt(hdcCalMem, 0, 0, xSize, ySize, hdcCalendar, 0, 0, SRCCOPY);

	// Copy the bitmap to the clipboard
	OpenClipboard(hDlg);
	EmptyClipboard();
	SetClipboardData(CF_BITMAP, hCalBitmap);
	CloseClipboard();
}
/*****************************************************************************/
