/***************************************************************************/
/*                                                                         */
/*        Linux Console Program to Test libfrequenc.so Library             */
/*                                                                         */
/*                                                                         */
/*  Revision record:                                                       */
/*	12/07/17    Created                                 		           */
/*                                                                         */
/* (c) 2017  W. Riley  Hamilton Technical Services  All Rights Reserved    */
/*                                                                         */
/***************************************************************************/

// Include
#include "frequenc.h"

// Macro for FrequenC library version number
#define FREQUENC_DLL_VERSION_NUMBER 300

int main(void)
{
	// Local variables

	// Create a set of classic freq test data as a double array
	// See NBS Monograph 140 Annex8.E
	// This array includes header info: # pts, analy start, analy end
	// and 9 freq data points
	double fDat[] = {9, 1, 9, 892, 809, 823, 798, 671, 644, 883, 903, 677};
	
	int nNum=0;			// # analysis points
	int nVer=0;			// FrequenC version code
	double fSig=1.0;	// Sigma

	// Show heading
	printf("FrequenC Library Example Program\n\n");

	// Get FrequenC version #
	nVer=CheckFrequenC();

	// Show FrequenC version
	printf("FrequenC Version = %3.2f\n", nVer/100.);

	// Call the Allan deviation function for test data array
	// Expect 8 analysis points and ADEV=91.22945
	// Similar calls can be used to try other analysis functions
	nNum=CalcFreqSigma(fDat, &fSig);
	
	// Check for valid result//
	if(nNum==-1) // Error
	{
		// Show error message
		printf("\nError in CalcFreqSigma()\n\n");
		
		// End program
		return 1;
	}
	else // OK
	{
		// Show results message
		printf("\nAllan Deviation Results:\n");

		// Display the # analysis points
		printf("# Analysis Points= %d\n", nNum);

		// Show sigma result
		printf("Sigma = %e\n\n", fSig);

		// Done
		return 0;
	}
}
