/*****************************************************************************/
/*                                                                           */
/*   Vibra2.c                                                                */
/*                                                                           */
/*   Dialog box procedure & related functions for Vibra program              */
/*                                                                           */
/*   (c) 1999  W. Riley  Hamilton Technical Services  All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

// Includes
#include "Vibra.h"
#include "resource.h"

/*****************************************************************************/
/*                                                                           */
/*                   Vibra Dialog Procedure                                  */
/*                                                                           */
/*      Revision record:                                                     */
/*          06/07/99    Created from Simple2.c                               */
/*			05/20/08	Added Sine FM type                                   */
/*                                                                           */
/*****************************************************************************/

BOOL CALLBACK VibraDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	// Local macros for calc type
	#define SINE	0
	#define RANDOM	1
	#define FM		2

    // Local variables
	char szBuffer[BUFFER_SIZE];
	static BOOL bType=SINE;
	static double fMHz=10;
	static double fGamma=1e-9;
	static double fG=1.0;
	static double fFreq=100.0;
	static double fL=0.0;

    // Dialog box message switch statement
    switch(msg)
    {
        case WM_INITDIALOG:
        {
			// Set the application system menu icon
			SetClassLong(hDlg, GCL_HICON, (LONG)hIcon);

			// Set the vib type button states
			Button_SetCheck(GETHCTL(IDC_VIBRA_SINE), !bType);
			Button_SetCheck(GETHCTL(IDC_VIBRA_RAND), bType);

			// Call the setup routine
			PostMessage(hDlg, WM_COMMAND, GET_WM_COMMAND_MPS(IDC_VIBRA_SETUP,
				NULL, 0));

			// Calc
			PostMessage(hDlg, WM_COMMAND, GET_WM_COMMAND_MPS(IDC_VIBRA_CALC,
				NULL, 0));

            return TRUE;
        }

 		case WM_COMMAND:
        {
            switch(GET_WM_COMMAND_ID(wParam, lParam))
            {
		       case IDC_VIBRA_HELP:
				{
					TRACE_STR("No Help File Available.");
					return 0;
				}

		       case IDC_VIBRA_SETUP:
				{
					// Set up the dialog box labels for the current vib type
					if(bType==SINE)
					{
						ShowWindow(GETHCTL(IDC_VIBRA_GAMMA), SW_SHOWNORMAL);
						SetWindowText(GETHCTL(IDC_VIBRA_GAMMALABEL),
							"Ga&mma, 1/g:");
						SetWindowText(GETHCTL(IDC_VIBRA_VIB),
							"Vibration");
						SetWindowText(GETHCTL(IDC_VIBRA_GLABEL),
							"&g, peak:");
						SetWindowText(GETHCTL(IDC_VIBRA_FVIBLABEL),
							"Vib Freq, &Hz:");
						SetWindowText(GETHCTL(IDC_VIBRA_DBCLABEL),
							"L(f), dBc:");
						SetWindowText(GETHCTL(IDC_VIBRA_RESULTS),
							"Spurious Level");
					}
					else if(bType==RANDOM)// Random
					{
						ShowWindow(GETHCTL(IDC_VIBRA_GAMMA), SW_SHOWNORMAL);
						SetWindowText(GETHCTL(IDC_VIBRA_GAMMALABEL),
							"Ga&mma, 1/g:");
						SetWindowText(GETHCTL(IDC_VIBRA_VIB),
							"Vibration");
						SetWindowText(GETHCTL(IDC_VIBRA_GLABEL),
							"&g/Hz:");
						SetWindowText(GETHCTL(IDC_VIBRA_FVIBLABEL),
							"Vib Freq, &Hz:");
						SetWindowText(GETHCTL(IDC_VIBRA_DBCLABEL),
							"L(f), dBc/Hz:");
						SetWindowText(GETHCTL(IDC_VIBRA_RESULTS),
							"Phase Noise");
					}
					else // FM
					{
						ShowWindow(GETHCTL(IDC_VIBRA_GAMMA), SW_HIDE);
						SetWindowText(GETHCTL(IDC_VIBRA_GAMMALABEL),
							"");
						SetWindowText(GETHCTL(IDC_VIBRA_VIB),
							"Modulation");
						SetWindowText(GETHCTL(IDC_VIBRA_GLABEL),
							"Hz, peak:");
						SetWindowText(GETHCTL(IDC_VIBRA_FVIBLABEL),
							"FM Rate, &Hz:");
						SetWindowText(GETHCTL(IDC_VIBRA_DBCLABEL),
							"L(f), dBc:");
						SetWindowText(GETHCTL(IDC_VIBRA_RESULTS),
							"FM SB Level");
					}


					return 0;
				}

		       case IDC_VIBRA_RESULT:
				{
					// Calc result for the current vib type
					if(bType==SINE)
					{
						fL=20*log10((fMHz*1e6*fG*fGamma)/(2*fFreq));
					}
					else if(bType==RANDOM)
					{
						fL=20*log10((fMHz*1e6*sqrt(2*fG)*fGamma)/(2*fFreq));
					}
					else // FM
					{
						fL=20*log10((fG)/(2*fFreq));
					}

					return 0;
				}

			   case IDC_VIBRA_SHOW:
				{
					// Show the carrier freq value
					sprintf(szBuffer, "%5.3f", fMHz);
					ShortenExp(szBuffer);
					SetWindowText(GETHCTL(IDC_VIBRA_MHZ), szBuffer);

					// Show the gamma value
					sprintf(szBuffer, "%5.4e", fGamma);
					ShortenExp(szBuffer);
					SetWindowText(GETHCTL(IDC_VIBRA_GAMMA), szBuffer);

					// Show the g value
					sprintf(szBuffer, "%5.4e", fG);
					ShortenExp(szBuffer);
					SetWindowText(GETHCTL(IDC_VIBRA_G), szBuffer);

					// Show the vib freq value
					sprintf(szBuffer, "%5.3f", fFreq);
					ShortenExp(szBuffer);
					SetWindowText(GETHCTL(IDC_VIBRA_FVIB), szBuffer);

					// Show the result value
					if(fL<0.0)
					{
						sprintf(szBuffer, "%5.2f", fL);
						ShortenExp(szBuffer);
					}
					else // Invalid result
					{
						strcpy(szBuffer, "Invalid");
					}
					
					SetWindowText(GETHCTL(IDC_VIBRA_DBC), szBuffer);

					return 0;
				}

				case IDC_VIBRA_MHZ:
				{
					// Clear result
					SetWindowText(GETHCTL(IDC_VIBRA_DBC), "");

                    // Respond to edit control when it loses the input focus
                    // unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (GetFocus()!=GETHCTL(IDC_VIBRA_CANCEL)))
                    {
                        // Get carrier freq
						GetWindowText(GETHCTL(IDC_VIBRA_MHZ), szBuffer,
							BUFFER_SIZE);
						if(atof(szBuffer)) // Ignore zero freq
						{
							fMHz=fabs(atof(szBuffer));
						}

						// Calc
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_VIBRA_CALC, NULL, 0));
					}

					return 0;
				}

				case IDC_VIBRA_GAMMA:
				{
					// Clear result
					SetWindowText(GETHCTL(IDC_VIBRA_DBC), "");

					// Respond to edit control when it loses the input focus
                    // unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (GetFocus()!=GETHCTL(IDC_VIBRA_CANCEL)))
                    {
                        // Get gamma
						GetWindowText(GETHCTL(IDC_VIBRA_GAMMA), szBuffer,
							BUFFER_SIZE);
						if(atof(szBuffer)) // Ignore zero gamma
						{
							fGamma=fabs(atof(szBuffer));
						}

						// Calc
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_VIBRA_CALC, NULL, 0));
					}

					return 0;
				}

				case IDC_VIBRA_G:
				{
					// Clear result
					SetWindowText(GETHCTL(IDC_VIBRA_DBC), "");

                    // Respond to edit control when it loses the input focus
                    // unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (GetFocus()!=GETHCTL(IDC_VIBRA_CANCEL)))
                    {
                        // Get g
						GetWindowText(GETHCTL(IDC_VIBRA_G), szBuffer,
							BUFFER_SIZE);
						if(atof(szBuffer)) // Ignore zero g
						{
							fG=fabs(atof(szBuffer));
						}

						// Calc
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_VIBRA_CALC, NULL, 0));
					}

					return 0;
				}

				case IDC_VIBRA_FVIB:
				{
					// Clear result
					SetWindowText(GETHCTL(IDC_VIBRA_DBC), "");

                    // Respond to edit control when it loses the input focus
                    // unless it loses it to the cancel button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (GetFocus()!=GETHCTL(IDC_VIBRA_CANCEL)))
                    {
                        // Get vib freq
						GetWindowText(GETHCTL(IDC_VIBRA_FVIB), szBuffer,
							BUFFER_SIZE);
						if(atof(szBuffer)) // Ignore zero vib freq
						{
							fFreq=fabs(atof(szBuffer));
						}

						// Calc
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_VIBRA_CALC, NULL, 0));
					}

					return 0;
				}

                // Respond to CR in an edit control
				// Note: No button is a DEFPUSHBUTTON
                case IDOK:
                {
                    // Does carrier freq edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_VIBRA_MHZ))
                    {
						// Send EN_KILLFOCUS message to carrier freq edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_VIBRA_MHZ, 0,
							EN_KILLFOCUS));
					}

					// Does gamma edit control have focus?
					if(GetFocus()==GETHCTL(IDC_VIBRA_GAMMA))
					{
						// Send EN_KILLFOCUS message to gamma edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_VIBRA_GAMMA, 0,
							EN_KILLFOCUS));
					}

					// Does g edit control have focus?
					if(GetFocus()==GETHCTL(IDC_VIBRA_G))
					{
						// Send EN_KILLFOCUS message to g edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_VIBRA_G, 0,
							EN_KILLFOCUS));
					}

					// Does vib freq edit control have focus?
					if(GetFocus()==GETHCTL(IDC_VIBRA_FVIB))
					{
						// Send EN_KILLFOCUS message to vib freq edit control
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_VIBRA_FVIB, 0,
							EN_KILLFOCUS));
					}
					
					return 0;
				}

				case IDC_VIBRA_CALC:
				{
					// Calc the results
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_VIBRA_RESULT, NULL, 0));

					// Show the results
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_VIBRA_SHOW, NULL, 0));

					return 0;
				}
				
				case IDCANCEL:
                case IDC_VIBRA_CANCEL:
                {
					// Close program
					PostQuitMessage(0);
					return 0;
                }

                case IDC_VIBRA_SINE:
                {
					// Set type
					bType=SINE;

					// Call the setup routine
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_VIBRA_SETUP, NULL, 0));

					// Calc
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_VIBRA_CALC, NULL, 0));

					return 0;
                }

                case IDC_VIBRA_RAND:
                {
					// Set type
					bType=RANDOM;

					// Call the setup routine
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_VIBRA_SETUP, NULL, 0));

					// Calc
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_VIBRA_CALC, NULL, 0));

					return 0;
                }

                case IDC_VIBRA_FM:
                {
					// Set type
					bType=FM;

					// Call the setup routine
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_VIBRA_SETUP, NULL, 0));

					// Calc
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_VIBRA_CALC, NULL, 0));

					return 0;
                }
                return TRUE;
               return TRUE;
            }
        }
        break;
    }

    return FALSE;
}

/*****************************************************************************/
/*                                                                           */
/*							ShortenExp()                                     */
/*                                                                           */
/*		Function to shorten all 3-digit exponents in a string                */
/*      Changes all numeric value formats of passed numeric value string     */
/*		from 3 digits to 2 digits                                            */
/*                                                                           */
/*      Parameters:                                                          */
/*      	szString		Numeric value string                             */
/*                                                                           */
/*      Return:                                                              */
/*			None	(void)                                                   */
/*                                                                           */
/*		Revision record:                                                     */
/*			01/16/98	Created                                              */
/*			06/09/99	Imported from STABLE_U.C                             */
/*                                                                           */
/*****************************************************************************/
void WINAPI ShortenExp( char *szString)
{
	// Local variables
    int i;		// Main index
	int j;		// Secondary index
	int len;	// String length

	// Find string length
	len=strlen(szString);
	
	// Scan for next 'e' in the string followed by a + or - and then a zero
	// Looking for pattern e+0dd or e-0dd
	// Scan stops 4 chars before end of string
	for(i=0; i<len-4; i++)
	{
		if( (szString[i]=='e') && (szString[i+1]=='+' || szString[i+1]=='-') &&
			(szString[i+2]=='0') )
		{
			// Original format is e+0dd or e-0dd
			// Want to eliminate the leading exp digit (always zero)
			// Modified format will be e+dd or e-dd
			// Shift up the string chars (and EOS) after the unwanted 0
			for(j=i+2; j<len; j++)
			{
				szString[j]=szString[j+1];
			}
			len--;
		}
	}
}

/*****************************************************************************/
